import requests
import xml.etree.ElementTree as ET
from string import Template
from datetime import datetime
import pytz

def remove_tags(txt):
    import re
    clean = re.compile('<.*?>')
    return re.sub(clean, '', txt)

def parse(url, limit=5):
    i = []
    r = requests.get(url)
    root = ET.fromstring(r.content)
    channel = root.find('channel')
    for n, item in enumerate(channel.findall('item'), start=1):
        if n > limit: break

        # Some feeds don't have <title>.
        # In that case, a truncated description is used.
        try:
            title = item.find('title').text
        except:
            title = item.find('description').text[0:25]+'…'

        # If a <item> doesn't havea <pubDate>, skip it
        try:
            date = item.find('pubDate').text
        except:
            continue
        i.append({
            'title': title, #item.find('title').text,
            'link': item.find('link').text,
            'date': date, #item.find('pubDate').text
            'isodate': datetime.strptime(date, '%a, %d %b %Y %H:%M:%S %z').isoformat()
            })

    return i

books = parse('https://gmgall.net/books/index.xml')
songs = parse('https://lfm.xiffy.nl/GMGall', limit=10)
posts = parse('https://gmgall.net/blog/index.xml')
toots = parse('https://bolha.us/@gmgall.rss')
#beers = parse('http://ctrl-c.club/~gmgall/feeds/gmgall_untappd.xml')

html_li = Template('<li><a href="$link">$title</a> <time datetime="$isodate">$date</time></li>')
books_lis = []
songs_lis = []
posts_lis = []
toots_lis = []

for book in books:
    books_lis.append(html_li.substitute(book))

for song in songs:
    songs_lis.append(html_li.substitute(song))
	
for post in posts:
    posts_lis.append(html_li.substitute(post))
	
for toot in toots:
    toot['title'] = remove_tags(toot['title'])
    toots_lis.append(html_li.substitute(toot))
	

html_begin = '''
<!DOCTYPE html>
<html lang="en-us">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <meta http-equiv="X-UA-Compatible" content="IE=edge">
  <style>body{font-family: "Ubuntu Mono", monospace;}</style>
  <title>~gmgall on ^C</title>
  <meta name="description" content="My little corner on ctrl-c.club">
  <meta name="author" content="gmgall">
</head>
<body>
  <main>
    <h1>gmgall on ctrl-c.club</h1>
    <p>So, you found me :-)</p>
    <hr>
    <p>🆕 I generate RSS feeds for pages here on <a href="https://ctrl-c.club/">ctrl-c.club</a> that don't have one. Check out at <a href="/~gmgall/feeds">~gmgall/feeds</a>.</p>
    <p>They are updated once a day.</p>
    <p>Found a broken one? Send me an email: <code>gmgall at ctrl-c.club</code></p>
    <p><b>TODO: </b>Generate an OPML file with all feeds.</p>
    <hr>
    <p>Below are some things I've been doing:</p>
    <p>Last 5 books read:</p>
    <ul>
'''

html = [ html_begin ]
html.extend(books_lis)
html.extend(['</ul>', '<p>Last 10 songs heard:</p>', '<ul>'])
html.extend(songs_lis)
html.extend(['</ul>', '<p>Last 5 posts on my blog:</p>', '<ul>'])
html.extend(posts_lis)
html.extend(['</ul>', '<p>Last 5 toots on Mastodon:</p>', '<ul>'])
html.extend(toots_lis)

html_end_template = Template('''
    </ul>
    <hr>
    <footer>Last update: $update</footer>
  </main>
</body>
</html>
''')

BRA = pytz.timezone('America/Sao_Paulo')
html_end = html_end_template.substitute(update = datetime.now().astimezone(BRA).strftime('%a, %d %b %Y %H:%M:%S %z'))

html.append(html_end)

with open('/home/gmgall/public_html/index.html', 'w') as f:
    f.write('\n'.join(html))
